<?php
/*
+--------------------------------------------------------------------------+
|   RevokeBB 1.0											               |
|   ========================================                               | 
|   author: denadai2 <denadai2@gmail.com>                                  |
|   copyright (c) 2006 RevokeSoft                                          |
|   http://www.revokesoft.net 											   |
|   license: http://opensource.org/licenses/gpl-license.php 			   |
|   GNU Public License                                					   |
|   ========================================							   |
+--------------------------------------------------------------------------+


*/

class Thread
	{
	
	/**
    * array of all core libraries
    * @access Public
    * @var array()
    */
	var $classes;
	
	/**
    * array of thread informations
    * @access Public
    * @var array()
    */
	var $thread;
	
	/**
    * Thread Constructor
    *
    * @param array classes, optional int tid
    * @author denadai2 <denadai2@gmail.com>
    * @since v 0.1
    * @access Public
    * @return Void
    */
	
	function Thread(& $classes, $tid=0)
		{
		$this->classes= & $classes;
		if($tid!=0)
			{
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('fetch_thread_by_id', $tid));
			$this->thread=$this->classes->db->fetch_array($query);
				
			}
		}
	
	/**
    * build new thread
    *
    * @param string $title, string $text, int $pool, int $forumid, int $userid, int $sticky
    * @author denadai2 <denadai2@gmail.com>
    * @since v 0.1
    * @access Public
    * @return array()
    */
	function new_thread($title, $text, $pool, $forumid, $info_user, $sticky, $bbcode_check)
		{
		
		//---------------------------------
		// Insert Thread
		//---------------------------------
		
		$dateline=time();
		
		$query = $this->classes->db->execQuery($this->classes->db->sql_build('new_thread', array($title, $info_user['user_id'], $info_user['user_nick'], $forumid, $info_user['user_nick'], $info_user['user_id'], $dateline, $dateline)));
		
		//---------------------------------
		// Insert Thread's Post
		//---------------------------------
		$thread_id=$this->classes->db->insert_id();
		$postid=$this->new_post($thread_id, '', $text, $info_user, 0, $bbcode_check);
		
		
		$query = $this->classes->db->execQuery($this->classes->db->sql_build('thread_firstpost_update', array($postid, $thread_id) ) );
		
		$query = $this->classes->db->execQuery($this->classes->db->sql_build('thread_threadcount_update', $forumid ) );
		//---------------------------------
		// Increase forum's threads count
		//---------------------------------
		$query=$this->classes->db->execQuery($this->classes->db->sql_build('board_threads_update', ($this->classes->_config['board_threads']+1) ));

		
		return array('thread_id' => $thread_id, 'post_id' => $postid );
		}
		
	/**
    * Text parsing for posts
    *
    * @param string $string
    * @author denadai2 <denadai2@gmail.com>
    * @since v 0.1
    * @access Public
    * @return string
    */
	function parse_text($string)
		{
		
		$patterns = array(
	
							'`\[b\](.+?)\[/b\]`is',
							'`\[i\](.+?)\[/i\]`is',
							'`\[u\](.+?)\[/u\]`is',
							'`\[strike\](.+?)\[/strike\]`is',
							'`\[color=#([0-9]{6})\](.+?)\[/color\]`is',
							'`\[email\](.+?)\[/email\]`is',
							'`\[img\](.+?)\[/img\]`is',
							'`\[url=([a-z0-9]+://)([\w\-]+\.([\w\-]+\.)*[\w]+(:[0-9]+)?(/[^ \"\n\r\t<]*?)?)\](.*?)\[/url\]`si',
							'`\[url\]([a-z0-9]+?://){1}([\w\-]+\.([\w\-]+\.)*[\w]+(:[0-9]+)?(/[^ \"\n\r\t<]*)?)\[/url\]`si',
							'`\[url\]((www|ftp)\.([\w\-]+\.)*[\w]+(:[0-9]+)?(/[^ \"\n\r\t<]*?)?)\[/url\]`si',
							'`\[quote=&quot;(.*?)&quot;\](.+?)\[/quote\]`is',
							'`\[indent](.+?)\[/indent\]`is',
							'`\[size=([1-6]+)\](.+?)\[/size\]`is',
							"#\[code(=(.*?))?\](.*?)\[/code\]#se",
							'`\[quote\](.+?)\[/quote\]`is'
	
						);
		
		$replaces =  array(
		
							'<strong>\\1</strong>',
							'<em>\\1</em>',
							'<span style="border-bottom: 1px dotted">\\1</span>',
							'<strike>\\1</strike>',
							'<span style="color:#\1;">\2</span>',
							'<a href="mailto:\1">\1</a>',
							'<img src="\1" alt="" style="border:0px;" />',
							'<a href="\1\2">\6</a>',
							'<a href="\1\2">\1\2</a>',
							'<a href="http://\1">\1</a>',
							'</p><div class="board-topic-quote-title">Quote $1</div><div class="board-topic-quote"><em>\2</em></div><p>',
							'<pre>\\1</pre>',
							'<h\1>\2</h\1>',
							"'$2<div class=\"board-topic-code-title\">Code</div>
			<div class=\"board-topic-code\">'.highlight_string(str_replace('<br />','',stripslashes('$3')), true).'</div>'",
							'<div class="board-topic-quote-title">Quote</div><div class="board-topic-quote"><em>$1</em></div>'
							);

		$string = preg_replace($patterns, $replaces , $string);
		return $string;
		}
		

	/**
    * prepare the text for insert
    *
    * @param string $text
    * @author denadai2 <denadai2@gmail.com>
    * @since v 1.0
    * @access Public
    * @return String
    */		
	function prepare_post($text, $bbcode_check=0)
	{
		
		if($this->classes->_config['wysiwyg']=='0' || $bbcode_check==1)
		{
			$text=$this->parse_text($text);
			$text=nl2br($text);
		}
		else
		{
		$text = html_entity_decode($text);
		
			$patterns = array('`\[quote=&quot;(.*?)&quot;\](.+?)\[/quote\]`is',
							"#\[code(=(.*?))?\](.*?)\[/code\]#se",
							'`\[quote\](.+?)\[/quote\]`is');
		
			$replaces =  array('<div class="board-topic-quote-title">Quote $1</div><div class="board-topic-quote"><em>\2</em></div>',
								"'$2<div class=\"board-topic-code-title\">Code</div>
				<div class=\"board-topic-code\">'.highlight_string(str_replace('<br />','',stripslashes('$3')), true).'</div>'",
				'<div class="board-topic-quote-title">Quote</div><div class="board-topic-quote"><em>$1</em></div>');

			$text = preg_replace($patterns, $replaces , $text); 
		}
		//---------------------------------
		// XSS FOBIAAA!!!!
		//---------------------------------			 
		$text = preg_replace('#</*(applet|meta|xml|blink|link|style|script|embed|object|iframe|frame|frameset|ilayer|layer|bgsound|title|base)[^>]*>#i',"",$text);			 
		$text = preg_replace('#(<[^>]+[\s\r\n\"\'])(on|xmlns)[^>]*>#iU',"$1>",$text);
		$text = preg_replace('#([a-z]*)[\x00-\x20]*=[\x00-\x20]*([\`\'\"]*)[\\x00-\x20]*j[\x00-\x20]*a[\x00-\x20]*v[\x00-\x20]*a[\x00-\x20]*s[\x00-\x20]*c[\x00-\x20]*r[\x00-\x20]*i[\x00-\x20]*p[\x00-\x20]*t[\x00-\x20]*:#iU','$1=$2nojavascript...',$text);
		$text = preg_replace('#([a-z]*)[\x00-\x20]*=([\'\"]*)[\x00-\x20]*v[\x00-\x20]*b[\x00-\x20]*s[\x00-\x20]*c[\x00-\x20]*r[\x00-\x20]*i[\x00-\x20]*p[\x00-\x20]*t[\x00-\x20]*:#iU','$1=$2novbscript...',$text);
		$text = preg_replace('#([a-z]*)[\x00-\x20]*=[\x00-\x20]*([\`\'\"]*)[\\x00-\x20]*j[\x00-\x20]*a[\x00-\x20]*v[\x00-\x20]*a[\x00-\x20]*s[\x00-\x20]*c[\x00-\x20]*r[\x00-\x20]*i[\x00-\x20]*p[\x00-\x20]*t[\x00-\x20]*:#iU','$1=$2nojavascript...',$text);
		$text = preg_replace('#([a-z]*)[\x00-\x20]*=([\'\"]*)[\x00-\x20]*v[\x00-\x20]*b[\x00-\x20]*s[\x00-\x20]*c[\x00-\x20]*r[\x00-\x20]*i[\x00-\x20]*p[\x00-\x20]*t[\x00-\x20]*:#iU','$1=$2novbscript...',$text);
	
		//$text = $this->limit_strlen($text);			
		$text = $this->removeEvilTags($text);
		$text=stripslashes(stripslashes($text));
		
		/* LINK RECOGNIZE nofollow...
		if(preg_match_all('<\s*a\s+[^>]*href\s*=\s*[\"\']?([^\"\' >]+)[\"\' >]', $text, $links)
		{
		
		}*/
		
		/*if (preg_match_all ('/img[\s]+[^>]*?src[\s]?=[\s\"\']+(.*?)[\"\']+.*?>/', $text, $regs)) 
		{
			if(is_array($regs))
			{
			require 'inc/class_gd_ver.php';
			$gd= new gd_handler();
			$num=count($regs[1]);
			for($i=0;$i<$num;$i++)
			{
				list($width, $height, $type, $attr) = getimagesize($regs[1][$i]);
				if($width>400 || $height>300)
				{
					$extension=explode('.',$regs[1][$i]);
					$name=time().'.'.$extension[count($extension)-1];
					$gd->create_thumb($regs[1][$i], $name);
					$temp=$regs[0][$i];
					$regs[0][$i]=str_replace($regs[1][$i], 'uploads/images/thumb_'.$name, $regs[0][$i]);
					//add url of big image
					$text=str_replace('<'.$temp, '<a href="uploads/images/'.$regs[1][$i].'"><'.$regs[0][$i].'</a>', $text);
				}
			}
			
			}
		
			
		}*/
		
		$text=addslashes($text);
		
		return $text;
		}
		
		/**
		* build new post
		*
		* @param int $topicid, string $title, string $text
		* @author denadai2 <denadai2@gmail.com>
		* @since v 0.1
		* @access Public
		* @return int
		*/		
		function new_post($topicid, $title, $text, $info_user, $new_post=1, $bbcode_check)
			{
			if(!$this->thread_exists($topicid))
				$this->classes->error->simple_error('generic');
				
			
			//---------------------------------
			// Increase post count
			//---------------------------------
			if( $info_user['user_id'] > 0 )
				$query = $this->classes->db->execQuery($this->classes->db->sql_build('user_postcount_update', $info_user['user_id'] ) );
			
			//---------------------------------
			// Insert post
			//---------------------------------
			$dateline=time();
			
			
			$text = $this->prepare_post($text, $bbcode_check);


			$query = $this->classes->db->execQuery($this->classes->db->sql_build('insert_post', array( $topicid, $info_user['user_id'], $text, $_SERVER['REMOTE_ADDR'], $dateline, $info_user['user_nick']  )));
			
			$return=$this->classes->db->insert_id();
			
			
			
			//---------------------------------
			// Increase forum's post count
			//---------------------------------
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('fetch_title_fid_by_tid', $topicid));
			$threadinfo= $this->classes->db->fetch_array($query);
			
			
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('update_forum_stats', array($return, $info_user['user_nick'], $info_user['user_id'], $threadinfo['title'], $topicid, $dateline, $threadinfo['forumid'])) );
			
			
			if($new_post==1)
				$query = $this->classes->db->execQuery($this->classes->db->sql_build('update_thread_stats', array($info_user['user_id'], $info_user['user_nick'], $dateline, $topicid)));
				
			
			//---------------------------------
			// Increase forum's general post count
			//---------------------------------		
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('increase_board_replies', ($this->classes->_config['board_replies']+1) ));	
			
			
			$this->thread_subcriber_email($topicid);

			$this->classes->cache->RemoveFromCache('sql_index_forum-0');
			$prec = $this->classes->skin->cache;
			$this->classes->skin->cache = true;
			$this->classes->skin->clear_all_cache();
			$this->classes->skin->cache = $prec;
			
			return $return;
			}
			
		/**
		* close a thread
		*
		* @param int $thread_id
		* @author denadai2 <denadai2@gmail.com>
		* @since v 0.1
		* @access Public
		* @return Void
		*/	
		function close_thread($thread_id)
			{
			if(!$this->thread_exists($thread_id))
				$this->classes->error->simple_error('no_t_exists');
			
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('update_closed_thread', $thread_id));
			}
			
		/**
		* pin a thread
		*
		* @param int $thread_id
		* @author denadai2 <denadai2@gmail.com>
		* @since v 0.1
		* @access Public
		* @return Void
		*/	
		function pin_thread($thread_id)
		{
			if(!$this->thread_exists($thread_id))
				$this->classes->error->simple_error('no_t_exists');
			
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('update_sticky_thread', $thread_id));
		}
			
		/**
		* unpin a thread
		*
		* @param int $thread_id
		* @author denadai2 <denadai2@gmail.com>
		* @since v 0.1
		* @access Public
		* @return Void
		*/	
		function unpin_thread($thread_id)
		{
			if(!$this->thread_exists($thread_id))
				$this->error->simple_error('no_t_exists');
			
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('update_unsticky_thread', $thread_id));
		}
			
		/**
		* edit a post
		*
		* @param int $thread_id
		* @author denadai2 <denadai2@gmail.com>
		* @since v 0.1
		* @access Public
		* @return Void
		*/	
		function edit_post($post_id, $text)
		{
			/*if(!$this->thread_exists($post_id))
				$this->classes->error->simple_error('no_t_exists');*/
			
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('update_edit_post', array($this->prepare_post($text), $post_id) ));
		}
						
		/**
		* Check if a thread exists
		*
		* @param int $thread_id
		* @author denadai2 <denadai2@gmail.com>
		* @since v 0.1
		* @access Public
		* @return bool
		*/	
		function thread_exists($thread_id)
		{
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('thread_exists', $thread_id));
			
			return($this->classes->db->num_rows($query)==1);
		}
			
		/**
		* fetch posts having topic id
		*
		* @param int $tid
		* @author denadai2 <denadai2@gmail.com>
		* @since v 0.1
		* @access Public
		* @return Void
		*/			
		function fetch_posts_by_tid($tid, $start, $last_10=0) //$last_10 need in newpost
		{
			if($last_10==0)
				$query = $this->classes->db->execQuery($this->classes->db->sql_build('fetch_posts_by_tid', array($tid,$start))); 
			else
				$query = $this->classes->db->execQuery($this->classes->db->sql_build('fetch_posts_by_tid_last_10', $tid));   
			
			$conta=1;
			
			//class uploads for type transformation
			require 'inc/class_uploads.php';
			
			while($vett=$this->classes->db->fetch_array($query))
			{
				
				$vett['dateline']=gmdate('d F Y H:i',$vett['dateline']);
				
				$this->classes->skin->content.='<modcp>'.intval($_SESSION['modcp']).'</modcp>';
				
				$vett['title'] = stripslashes($vett['title']);
				$vett['numpost'] = $conta++;
				$vett['user_yahoo'] = stripslashes($vett['user_yahoo']);
				$vett['user_msn'] = stripslashes($vett['user_msn']);
				$vett['user_website'] = stripslashes($vett['user_website']);
				$vett['user_sign'] = stripslashes($vett['user_sign']);
				$vett['text'] = stripslashes($vett['text']);
				$vett['image'] = stripslashes($vett['image']);
				$vett['online'] = intval((($vett['lastaction']+(15*60))>time()));
				$vett['thread_id'] = $tid;
				
				if($vett['upload']!='1')
				{
					
					$upload = new File_Upload ( $this->classes );
					
					$vett['upload_type'] = $upload->filetypes2icons($vett['upload_type']);
				}
				
				
				$all_vett[]=$vett;
			}
			$this->classes->skin->assign('posts', $all_vett);
			
		}
		
		/**
		* Check if a thread exists
		*
		* @param int $thread_id
		* @author denadai2 <denadai2@gmail.com>
		* @since v 0.1
		* @access Public
		* @return bool
		*/	
		function post_text_edit($post_id, $text)
		{
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('update_post_text', array($text, $post_id) ));
			
			return($query==true);
		}
			
		/**
		* Check if a thread exists
		*
		* @param int $thread_id
		* @author denadai2 <denadai2@gmail.com>
		* @since v 0.1
		* @access Public
		* @return bool
		*/	
		function fetch_post_text($post_id)
		{
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('fetch_post_text_pid', $post_id ));
			$vett=$this->classes->db->fetch_array($query);
			
			return($vett['text']);
		}
			
		/**
		* Send notifications emails to threads subscribers
		*
		* @param int $tid
		* @author denadai2 <denadai2@gmail.com>
		* @since v 1.0
		* @access Public
		* @return none
		*/				
		function thread_subcriber_email($tid)
		{
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('fetch_thread_subscriber', array($tid, $this->classes->user->info_user['user_id'] ) ) );
			$headers = "Bcc:";
			
			$last = '';
			
			if($this->classes->db->num_rows($query)>0)
			{
				while($vett=$this->classes->db->fetch_array($query))
				{
					$last = $vett;
					
					if($last!='')
						$headers = ' , ';
					
					$headers .= $vett['user_email'];	
				}
				
				$headers = "From: ".$this->classes->_config['admin_email']."\n";
				$headers .= "Return-Path: ".$this->classes->_config['admin_email']."\n";
				@mail( $last['user_email'] ,$this->classes->lang->lang_all['showthread']['main']['replyed'] . $last['title'], $this->classes->lang->lang_all['showthread']['main']['sub_user'] .$last['title']." \n\n\n".$this->classes->_config['board_domain'].$this->classes->_config['board_url'].'index.php?act=showthread&tid='.$last['thread_id'].'&view=lastposts'."\n\n", $headers );
				
				$query = $this->classes->db->execQuery($this->classes->db->sql_build('update_thread_subscriber', array($tid, $this->classes->user->info_user['user_id'] ) ) );
			}
			
		}
		
		
		/**
		* Subscribe an user to a thread
		*
		* @param int $tid
		* @author denadai2 <denadai2@gmail.com>
		* @since v 1.0
		* @access Public
		* @return none
		*/				
		function thread_subscribe($tid)
		{
			if($this->classes->user->info_user['user_id'] > 0)
			{
				$query = $this->classes->db->execQuery($this->classes->db->sql_build('unsubscribe_thread_by_uid', array($this->classes->user->info_user['user_id'], $tid)  ) );
				
				$query = $this->classes->db->execQuery($this->classes->db->sql_build('subscribe_user_to_thread', array($tid, 1, $this->classes->user->info_user['user_id'], 1)  ) );		
			}
			
		}
		
		/**
		* Strip forbidden tags and delegate tag-source check to removeEvilAttributes()
		*
		* @param string $source
		* @author tREXX <www.trexx.ch>
		* @since v 0.1
		* @access Public
		* @return string
		*/	
		function removeEvilTags($source)
		{
		   
		   return preg_replace('/<(.*?)>/ie', "'<'.$this->removeEvilAttributes('\\1').'>'", $source);
		}
		
		/**
		* Strip forbidden attributes from a tag
		*
		* @param string $source
		* @author tREXX <www.trexx.ch>
		* @since v 0.1
		* @access Public
		* @return string
		*/
		function removeEvilAttributes($tagSource)
		{
		   $stripAttrib = 'javascript:|onclick|ondblclick|onmousedown|onmouseup|onmouseover|'.
					   'onmousemove|onmouseout|onkeypress|onkeydown|onkeyup';
		   return stripslashes(preg_replace("/$stripAttrib/i", 'forbidden', $tagSource));
		}
		
		
		/**
		* limit the strings in a portion of text
		*
		* @param String $text
		* @author denadai2 <denadai2@gmail.com>
		* @since v 1.0
		* @access Public
		* @return String
		*/			
		function limit_strlen($text)
		{
			$array = explode(' ',$text);
			$num=count($array);
			for($i=0;$i<$num;$i++)
				if(strlen($array[$i])>20)
					$array[$i]=substr($array[$i],0,19);
			return implode(' ',$array);
		}

		/**
		* Delete a thread
		*
		* @param int $tid
		* @author denadai2 <denadai2@gmail.com>
		* @since v 1.0
		* @access Public
		* @return none
		*/		
		function delete_thread($tid)
		{
		
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('thread_delete', $tid)  );
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('thread_posts_delete', $tid) );
			
			$this->classes->cache->RemoveFromCache('sql_index_forum-0');
		$prec = $this->classes->skin->cache;
		$this->classes->skin->cache = true;
		$this->classes->skin->clear_all_cache();
		$this->classes->skin->cache = $prec;
		}

		/**
		* Delete a post
		*
		* @param int $tid
		* @param int $post_id
		* @author denadai2 <denadai2@gmail.com>
		* @since v 1.0
		* @access Public
		* @return none
		*/		
		function delete_post($tid, $post_id)
		{
		
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('delete_post_by_post_id', $post_id)  );
			
			$this->sync_thread($tid);
			
			$this->classes->cache->RemoveFromCache('sql_index_forum-0');
		$prec = $this->classes->skin->cache;
		$this->classes->skin->cache = true;
		$this->classes->skin->clear_all_cache();
		$this->classes->skin->cache = $prec;
		}
		
		/**
		* Move a thread
		*
		* @param int $tid, int $destforum
		* @author denadai2 <denadai2@gmail.com>
		* @since v 1.0
		* @access Public
		* @return none
		*/			
		function move_thread($tid, $destforum)
		{
		
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('thread_move', array($destforum,$tid))  );
			
			$this->classes->cache->RemoveFromCache('sql_index_forum-0');
		$prec = $this->classes->skin->cache;
		$this->classes->skin->cache = true;
		$this->classes->skin->clear_all_cache();
		$this->classes->skin->cache = $prec;
		}
		
		function sync_thread($tid)
		{
	
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('sync_thread',$tid));
			$temp=$this->classes->db->fetch_array($query);
			if($temp['user_id'] == '-1')
				$temp['user_nick'] = $temp['username'];
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('sync_thread_update', array($temp['user_nick'], $temp['user_id'], $tid)));
			
			if($this->sync_thread_count($tid)==1)
			{
				$query = $this->classes->db->execQuery($this->classes->db->sql_build('sync_thread_update', array($temp['user_nick'], $temp['user_id'], $tid)));
			}
		
		}
	
	
		function sync_thread_count($tid)
		{
	
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('sync_thread_count',$tid));
			$temp=$this->classes->db->fetch_array($query);
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('sync_thread_count_update', array($temp['posts'], $tid)));
			
			return $temp['posts'];
		}
		
		/**
		* Check if a 3d is visited
		* @param int tid
		* @author dierre <dierre@gmail.com> 
		* since v 1.0
		* @access Public
		* @return none
		*/	
		function update_views($tid)
		{
			$query = $this->classes->db->execQuery($this->classes->db->sql_build('update_visits', $tid));
		}
		

	
	}

	
?>