<?php
/******************************************************************************
 Pepper
 
 Developer		: Sebastian Mller
 Plug-in Name	: Vigraph
 
 http://www.cooch.de/category/vigraph/
 
 ******************************************************************************/

class graph3D {
		
	var $fontPath 	= array(
					  'monaco'	=> 'inc/graph/MONACO.TTF'
					  );	
	var $height;
	var $width;
	var $padding 	= array();					  
	var $image;
	var $colors		= array();	
	var $values		= array();
	var $maxHeight;
	var $maxNum;
	var $dayWidth;
	var $lineHeight;
	var $vpp;
			  
	function graph3D($width,$height) {
		$this->height 	= $height;
		$this->width	= $width;
		$this->padding	= array('top' => 0,'bottom' => 60,'sides' => 5);	
		$this->image	= imagecreatetruecolor($this->width,$this->height);
		$this->colors	= array(
					  'text' 	=> imagecolorallocate($this->image, 100, 100, 100),
					  'grey' 	=> imagecolorallocate($this->image, 220, 220, 220),
					  'white'	=> imagecolorallocate($this->image, 0xFF, 0xFF, 0xFF),
				      'color1'	=> imagecolorallocate($this->image, 0xD7, 0xEA, 0xB3),
					  'color2'	=> imagecolorallocate($this->image, 0xBD, 0xDC, 0x84),
					  'color3'	=> imagecolorallocate($this->image, 0x7B, 0x9F, 0x53),
				      'color4'	=> imagecolorallocate($this->image, 0x91, 0xB0, 0x68),
					  'color5'	=> imagecolorallocate($this->image, 0xB7, 0xD9, 0x78),
					  'color6'	=> imagecolorallocate($this->image, 0x9D, 0xCB, 0x48),
					  'color7'	=> imagecolorallocate($this->image, 0x83, 0xAF, 0x31),
					  'color8'	=> imagecolorallocate($this->image, 0xF0, 0xF7, 0xE2)
					  );			
		imagefill($this->image,0,0,$this->colors['white']);
	}	

	function dataInput($tNums,$tKeys,$bNums,$bKeys) {
		if (count($tNums) == count($bNums)) {
			foreach ($tNums as $id => $val) 
				$this->values['top'][$tKeys[$id]] = $val;
			foreach ($bNums as $id => $val) 
				$this->values['bot'][$bKeys[$id]] = $val;
			$this->setSettings();
		} else
			die("Wrong data input");
	}
	
	function setSettings() {
	
		$nums 				= count($this->values['top']);
		$this->maxHeight 	= ($this->height - $this->padding['bottom']) - 20;		
		$this->maxNum		= max($this->values['top']);
		$this->vpp			= $this->maxHeight / $this->maxNum;											
		$width				= ($this->width - (2 * $this->padding['sides'])) - 50;						
		$this->dayWidth		= $width / ($nums+1);														
		$this->lineHeight	= (int)($this->maxHeight / 10);		
	}
	
	function drawBeginning() {
		$this->_drawBackgroundLines();
		$this->_drawBottom();

		$this->writeTextForNumbersSide();		
	}
	
	function drawEnding($legend = 1) {
				$this->_drawSideLines();
		imageline($this->image,
				  $this->padding['sides'],
				  $this->height - $this->padding['bottom'] + 2,
				  $this->width -  $this->padding['sides'],
				  $this->height - $this->padding['bottom'] + 2,
				  $this->colors['text']);		// BottomLine	
		if($legend ==1)	
		$this->drawLegend($this->colors['color6'],$this->colors['color5']);
	}
	
	function show() {
		imagepng($this->image);	
	}
	
	function save($name) {
		imagepng($this->image,$name);	
	}
	
	function _drawBackgroundLines() {
		$i 	= 2;
		while($i <= 11) {
			if ((($i+1) % 2) == 0)
				$color = $this->colors['color1'];
			else
				$color = $this->colors['grey'];
			imageline($this->image,
					  $this->padding['sides'] + 22,
					  $this->height - $this->padding['bottom'] - ($i*$this->lineHeight) + 6,
					  $this->width  - $this->padding['sides']+2,
					  $this->height - $this->padding['bottom'] - ($i*$this->lineHeight) + 6,
					  $color);		// BackgroundLine
			imageline($this->image,
					  $this->padding['sides'] + 10,
					  $this->height - $this->padding['bottom'] - ($i*$this->lineHeight) + 17,
					  $this->padding['sides'] + 21,
					  $this->height - $this->padding['bottom'] - ($i*$this->lineHeight) + 6,
					  $color);		// leftside		
			imageline($this->image,
					  $this->width  - $this->padding['sides'] - 10,
					  $this->height - $this->padding['bottom'] - ($i*$this->lineHeight) + 18,
					  $this->width  - $this->padding['sides'] + 2,
					  $this->height - $this->padding['bottom'] - ($i*$this->lineHeight) + 6,
					  $color);		// rightside						  		  
			$i++;	
		}		
	}
	
	function _drawSideLines() {
		imageline($this->image,
				  $this->padding['sides'] + 10,
				  $this->height - $this->padding['bottom'] + 10,
				  $this->padding['sides'] + 10,
				  22,
				  $this->colors['text']);		// LeftLine front
				  
		imageline($this->image,
				  $this->width  - $this->padding['sides']  - 10,
				  $this->height - $this->padding['bottom'] + 10,
				  $this->width  - $this->padding['sides']  - 10,
				  23,
				  $this->colors['text']);		// RightLine front
				  
		imageline($this->image,
				  $this->padding['sides'] + 22,
				  $this->height - $this->padding['bottom'] - 10,
				  $this->padding['sides'] + 22,
				  11,
				  $this->colors['grey']);		// LeftLine back
				  
		imageline($this->image,
				  $this->width  - $this->padding['sides'] + 2,
				  $this->height - $this->padding['bottom'] - 10,
				  $this->width  - $this->padding['sides'] + 2,
				  11,
				  $this->colors['grey']);		// RightLine back
	}	
	
	function _drawBottom() {
		$i = -1;
		while($i <= 10) {
			imageline($this->image,
					  $this->padding['sides'] + 12 + $i,
					  $this->height - $this->padding['bottom'] - $i,
					  $this->width  - $this->padding['sides']  - 8 + $i,
					  $this->height - $this->padding['bottom'] - $i,
					  $this->colors['grey']);	
		 	$i++;	
		}	
	}
	
	function drawVerticalBlocks($text = true) {
		$daycount = 0;
		foreach ($this->values['bot'] as $key => $value) {
			$x1 = ($this->padding['sides'] + ($daycount * $this->dayWidth)) + 60;
			$y1 = $this->height - $this->padding['bottom'] -2;
			$x2 = ($this->padding['sides'] + ($daycount * $this->dayWidth)) + 60;
			$y2 = (($this->height - $this->padding['top']) - $this->padding['bottom']) - ($value * $this->vpp) - 2;
			if ($value > 0)
				$this->_drawVerticalBlock($x1,$y1,$x2,$y2,true,$this->colors['color6'],$this->colors['color7']);	
			if ($text == true)
				$this->writeTextForVerticalBlock($x1 - 10,$y1 + 5,$key);
			$daycount++;
		}
		
		$daycount = 0;
		foreach ($this->values['top'] as $key => $value) {
			$x1 = ($this->padding['sides'] + ($daycount * $this->dayWidth)) + 60;
			$y1 = $this->height - $this->padding['bottom'] - 2 - ($this->values['bot'][$key] * $this->vpp);
			$x2 = ($this->padding['sides'] + ($daycount * $this->dayWidth)) + 60;
			$y2 = (($this->height - $this->padding['top']) - $this->padding['bottom']) - ($value * $this->vpp) - 2;
			if ($value > 0)
				$this->_drawVerticalBlock($x1,$y1,$x2,$y2,true,$this->colors['color5'],$this->colors['color4']);
		
			$daycount++;
		}		
	}
	
	function drawLegend($color1,$color2,$text1 = 'Total',$text2 = 'Unique') {
		imagerectangle($this->image,($this->width/2)-80,$this->height-20,($this->width/2)+80,$this->height-5,$this->colors['grey']);
		imagerectangle($this->image,($this->width/2)-79 ,$this->height-19,($this->width/2)+79 ,$this->height-4,$this->colors['grey']);
		
		imagefilledrectangle($this->image,($this->width/2) - 15,$this->height - 16,($this->width/2) - 7 ,$this->height-8,$color1);		
		imagerectangle		($this->image,($this->width/2) - 15,$this->height - 16,($this->width/2) - 7 ,$this->height-8,$this->colors['grey']);

		imagefttext			($this->image,6,0,($this->width/2) + 23,$this->height - 8,$this->colors['text'],$this->fontPath['monaco'],$text1);
		imagefttext			($this->image,6,0,($this->width/2) - 55,$this->height - 8,$this->colors['text'],$this->fontPath['monaco'],$text2);

		imagefilledrectangle($this->image,($this->width/2) + 7,$this->height - 16,($this->width/2) + 15 ,$this->height-8,$color2);		
		imagerectangle		($this->image,($this->width/2) + 7,$this->height - 16,($this->width/2) + 15 ,$this->height-8,$this->colors['grey']);

	}
	
	function writeTextForVerticalBlock($x,$y,$text,$angle = "45",$size = "7") {
		imagefttext($this->image, $size, $angle, $x - 10, $y + 27, $this->colors['text'], $this->fontPath['monaco'], $text);	
	}
	
	function writeTextForNumbersSide() {
		$max = $this->maxNum;
		$u = 1;
		while($u <= 5) {
			$nums[$u] = (int)($u*($max/5));
			$u++;
		}
		$i = 1;
		$u = 1;
		while($i <= 9) {
			imagettftext($this->image,6,0,$this->padding['sides']+27,23+($i-1)*$this->lineHeight,$this->colors['color7'],$this->fontPath['monaco'],$nums[(6-$u)]);	
			$i = $i + 2;
			$u++;
		}
	}
	
	function _drawVerticalBlock($x1,$y1,$x2,$y2,$room,$color1,$color2) {
		imagefilledrectangle($this->image,$x2-5,$y2,$x1+5,$y1,$color1);
		if ($room == true) {
			$points[] = $x1+5; 
			$points[] = $y1;    
			$points[] = $x1+10;
			$points[] = $y1-5;
			$points[] = $x1+10;
			$points[] = $y2-5;
			$points[] = $x2;
			$points[] = $y2-5;
			$points[] = $x2-5;
			$points[] = $y2;
			$points[] = $x2+5;
			$points[] = $y2;						
			imagefilledpolygon($this->image,$points,6,$color2);
		}
	}
	
	
	
	
	
	
	function drawRow($text1 = 'Total') {
		/*imagerectangle($this->image,($this->width/2)-80,$this->height-20,($this->width/2)+80,$this->height-5,$this->colors['grey']);
		imagerectangle($this->image,($this->width/2)-79 ,$this->height-19,($this->width/2)+79 ,$this->height-4,$this->colors['grey']);
		
		imagefilledrectangle($this->image,($this->width/2) - 15,$this->height - 16,($this->width/2) - 7 ,$this->height-8,$color1);		
		imagerectangle		($this->image,($this->width/2) - 15,$this->height - 16,($this->width/2) - 7 ,$this->height-8,$this->colors['grey']);*/

		imagefttext			($this->image,9,0,$this->width /8,$this->height /2,$this->colors['text'],$this->fontPath['monaco'],$text1);

		/*imagefilledrectangle($this->image,($this->width/2) + 7,$this->height - 16,($this->width/2) + 15 ,$this->height-8,$color2);		
		imagerectangle		($this->image,($this->width/2) + 7,$this->height - 16,($this->width/2) + 15 ,$this->height-8,$this->colors['grey']);*/

	}
	
	
	
	
	
	
}